/*******************************
**			      **
**	   Keyboard.h	      **
**			      **
**  A set of useful routines  **
**  for controlling keyboard  **
**  settings and information. **
**			      **
**  (C) 1994 by Sean Werkema  **
**  Revision 2.0    7/4/1994  **
**			      **
*******************************/

#ifndef K_LANG
#ifdef __cplusplus
#define K_LANG "C"
#else
#define K_LANG
#endif
#endif

/************************************************************************
		      Version 2.0 data & functions		7/10/1994
************************************************************************/

/*
**  Function prototypes
*/

extern K_LANG int KeyASCII();
/*     Holds last ASCII value read from keyboard.  Changes with each read,
**     but does not change with each call.
*/
extern K_LANG int KeyScan();
/*     Holds last scan code value read from keyboard.  Changes with each read,
**     but does not change with each call.
*/
extern K_LANG int GetCursorStart(void);
/*     Returns value corresponding to top of cursor, usually 0-7.
*/
extern K_LANG int GetCursorEnd(void);
/*     Returns value corresponding to bottom of cursor, usually 0-7.
*/
extern K_LANG void SetCursorSize(int Start, int End);
/*     Adjusts cursor size to top and bottom values (usually from 0-7).
*/
extern K_LANG int GetCursorX(int page);
/*     Returns horizontal position of cursor, ranging from 1 to <scrn width>.
**     If page is -1, the currently displayed page will be used.
*/
extern K_LANG int GetCursorY(int page);
/*     Returns vertical position of cursor, ranging from 1 to <scrn height>.
**     If page is -1, the currently displayed page will be used.
*/
extern K_LANG void SetCursorPos(int page, int X, int Y);
/*     Moves cursor to specified coordinates, with the origin at 1.
**     If page is -1, the currently displayed page will be used.
*/
extern K_LANG int ReadKeyASCII(void);
/*     Waits for a key to be pressed, then returns the ASCII value of that
**     key.  KeyASCII and KeyScan are set to the key's ASCII value & scan code.
*/
extern K_LANG int ReadKeyScan(void);
/*     Waits for a key to be pressed, then returns the scan code of that
**     key.  KeyASCII and KeyScan are set to the key's ASCII value & scan code.
*/
extern K_LANG int KeyStatus(void);
/*     Returns 1 if a keypress is waiting, and 0 if no keypress is waiting.  If
**     1, KeyASCII and KeyScan are set to the key's ASCII value & scan code.
*/
extern K_LANG int KeyFlags(void);
/*     Returns keyboard flags: L Shf, R Shf, Ctrl, Alt, Caps, Num, Scrl, Ins.
*/
extern K_LANG int EReadKeyASCII(void);
/*     Waits for a key to be pressed, then returns the ASCII value of that
**     key.  KeyASCII and KeyScan are set to the key's ASCII value & scan code.
**     Same as ReadKeyASCII, except can detect extended keys.
*/
extern K_LANG int EReadKeyScan(void);
/*     Waits for a key to be pressed, then returns the scan code of that
**     key.  KeyASCII and KeyScan are set to the key's ASCII value & scan code.
**     Same as ReadKeyScan, except can detect extended keys.
*/
extern K_LANG int EKeyStatus(void);
/*     Returns 1 if a keypress is waiting, and 0 if no keypress is waiting.  If
**     1, KeyASCII and KeyScan are set to the key's ASCII value & scan code.
**     Same as KeyStatus, except can detect extended keys.
*/
extern K_LANG int EKeyFlags(void);
/*     Returns keyboard flags: L Shf, R Shf, L Ctrl, R Ctrl, L Alt, R Alt,
**     Caps, Num, Scrl, Ins, SysReq.
**     Same as KeyFlags, except can detect extended keys.
*/
extern K_LANG int IsEKeybd(void);
/*     Returns 0 if not an Extended Keyboard, 1 if is an Extended Keyboard.
*/

/************************************************************************
		      Version 2.0 data & functions		 7/4/1994
************************************************************************/

/*
**     Special keyboard control functions
*/

extern K_LANG void NewKeyboard(void);
/*     Initializes special keyboard data control functions.
**     Standard functions will not work when special functions are
**     active, but the special functions allow more versatile control
**     of keyboard data.  Accepts no values, returns no values.
*/
extern K_LANG void OldKeyboard(void);
/*     Deactivates special keyboard functions, allowing normal
**     functions to be used instead.  If special functions have
**     been activated, deactivate them before exiting program
**     or attempting to use normal functions.  Accepts no values,
**     returns no values.
*/
extern K_LANG unsigned int GetStroke(void);
/*     Special function that returns a scan code corresponding to
**     a key press or release, both of which are stored in a FIFO
**     buffer from which this function reads its information.  If
**     the upper byte of the returned integer contains 0xE0 the
**     key press/release was from a key on the Extended Keyboard
**     (Gray Insert/Delete/Home/End/PgUp/PgDn/Arrows/Enter/Slash or Right
**     Ctrl/Alt).  If bit 7 is set (high bit of low byte) the value
**     indicates a key release; if bit 7 is clear, the value indicates
**     a key press.  If the returned integer is 0, no keystrokes are
**     waiting.  Low 7 bits contain scan code corresponding to
**     the key, as per the following table:
**
**			Scan Code Table (Hexadecimal values given)
**     Ŀ ĿĿ Ŀ Ŀ
**     01  3B3C3D3E3F404142 43445758 1246E1
**     Esc F1F2F3F4F5F6F7F8 F9101112 PSSLPz
**        
**     Ŀ Ŀ Ŀ
**     2902030405060708090A0B0C0D2B0E 524748 4535374A
**     ` 1 2 3 4 5 6 7 8 9 0 - = \ - InHmPU NL/ * - 
**     Ĵ Ĵ Ĵ
**     0F 101112131415161718191AAB     534F49 4748494E
**     TabQ W E R T Y U I O P [ ]      DlEnPD 7 8 9   
**      1C   Ĵ+ 
**     3A  1E1F202122232425262728	    		 4B4C4D  
**     CapsA S D F G H J K L ; '                4 5 6   
**     Ĵ	 Ŀ	 Ĵ
**      2A   2C2D2E2F303132333435 36    	 48	 4F50511C
**      ShiftZ X C V B N M , . /  Shift 	  	 1 2 3   
**     Ĵ Ŀ ĴE 
**      1D   38 	39		38   1D   4B504D 52   53n 
**     Ctrl  Alt	Spacebar	Alt  Ctrl     0    . t 
**           
**     NOTE: The following keys' scan codes are preceded by a 0xE0:
**     Right Alt;  Right Ctrl;	Grey Arrows,/,Ins,Home,Del,End,PgUp,PgDn,Enter.
*/
extern K_LANG unsigned int TranASCII(int value);
/*     Translates a scan code into an unshifted (as if neither shift key
**     were held down) ASCII code.  Passed value is the same as that returned
**     by GetStroke, ReadKeyScan, or contained in KeyScan.  Returned value
**     is corresponding ASCII code.
*/
extern K_LANG unsigned int TranASCII2(int value);
/*     Translates a scan code into a shifted (as if either shift key were
**     held down) ASCII code.  Passed value is the same as that returned
**     by GetStroke, ReadKeyScan, or contained in KeyScan.  Returned value
**     is corresponding ASCII code.
*/
